;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc76e01745f455680bed3cee2836fa535 . "./code.cg2js/cg-acache.fasl")
(#xa9b267f3fcb7822e5566676e23b3b69b . "./code.cg2js/cg-base.fasl")
(#x6372afe266eec517674b1155515ed730 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x646a9164cc43a11e21e64cb59f174b33 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x55b25354a1b2af843cc79061c0ddb992 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x797c102700356b28e4b356657fab242d . "./code.cg2js/cg-button.fasl")
(#xc79ecc00f3b441e9f4eca5a493e4654e . "./code.cg2js/cg-calendar.fasl")
(#x1f37a013155d5b02f2792d2f2877638d . "./code.cg2js/cg-caret.fasl")
(#xb3e452dcb0cf9bd7f13757e933ba050a . "./code.cg2js/cg-chart-or-plot.fasl")
(#xd96329962ccf1204eb84544c0ec6f4c0 . "./code.cg2js/cg-chart-widget.fasl")
(#x0463724421e46836cb5817b46cdedcb6 . "./code.cg2js/cg-check-box.fasl")
(#xae6141a2e55a0a435071d089a9e410bb . "./code.cg2js/cg-choice-list.fasl")
(#xd009114b4ef0f25e164a611533000b19 . "./code.cg2js/cg-choose-printer.fasl")
(#xbaa5cf17014d444a72ed4e3fa7321c16 . "./code.cg2js/cg-class-grid.fasl")
(#xf7009c8650fa85f79f694586823a1e31 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x51d1adcd54d20f53bee85b4d9f93fede . "./code.cg2js/cg-class-support.fasl")
(#x9ccf87012670d8fd9d7cf7b78c24f89c . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xf7ae207107a75e4202b335b109de1dbd . "./code.cg2js/cg-clipboard-stack.fasl")
(#xa11dce148144ab42686ec519788d5585 . "./code.cg2js/cg-clipboard.fasl")
(#x6717aea4b64bafcbabd17ee4a3e0f46c . "./code.cg2js/cg-color-dialog.fasl")
(#x7d5ed43ec6e2483d58ccc8ec77bd2666 . "./code.cg2js/cg-combo-box.fasl")
(#xeda6d0278677090dd5bff308ee8c870a . "./code.cg2js/cg-common-control.fasl")
(#xff980c6aaae70151531570e30a02924e . "./code.cg2js/cg-comtab.fasl")
(#x12a1861aa0f08234f0d4f3299cc3fa4a . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x750f06eb162f7d2a1e5786744507ff74 . "./code.cg2js/cg-curve.fasl")
(#x673b3f3aca1b91cb66897571a6138f80 . "./code.cg2js/cg-dde-utils.fasl")
(#xf57637d691a4739d256d3e8a2aad3ff8 . "./code.cg2js/cg-dev.fasl")
(#xe0f11d8ca00ae4baf623fb80b7b98bb4 . "./code.cg2js/cg-dialog-item.fasl")
(#xf7091dd002671c1dc7fc0b58a0a02281 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xd2c658fbd02dd87facd10b84bd679065 . "./code.cg2js/cg-directory-dialog.fasl")
(#xc68640acea75bb123fb6021bc63d2c1e . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x50765ebe4a106bdf07418d27feb20c90 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x63fc50a699ef56af6ec7a35ac7029e4e . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf5ad215cfe06cdad38f66414f2640ad8 . "./code.cg2js/cg-drawable.fasl")
(#x97bdfb9f58edec1b4fb32d51454301be . "./code.cg2js/cg-dropping-outline.fasl")
(#x2c4d246c9d8f170704f36346f6f41887 . "./code.cg2js/cg-edit-in-place.fasl")
(#x667e46413eaa50e98105f59ee6e506e6 . "./code.cg2js/cg-editable-text.fasl")
(#x825c02dba0cf737e25c128436e268a4f . "./code.cg2js/cg-file-dialog.fasl")
(#x8f79854bb4d73cbcf88d400f4a627469 . "./code.cg2js/cg-fill-texture.fasl")
(#xe2f533e131028c2cf96f02f350ba3c9d . "./code.cg2js/cg-find-string-dialog.fasl")
(#x3ab68050597647df83add6f64a08c46e . "./code.cg2js/cg-font-dialog.fasl")
(#xb4586b6b719fd17840fc2147a8c17dcb . "./code.cg2js/cg-gesture-emulation.fasl")
(#x305d2af7bc5e9dff3c6c73bc4f337a3b . "./code.cg2js/cg-get-pixmap.fasl")
(#x21363130cc6ebc5d7b7a50aa6213a20c . "./code.cg2js/cg-get-position.fasl")
(#x858f4aafbd560095bcbfcf61422065aa . "./code.cg2js/cg-graphics-context.fasl")
(#x0e8f9985afd8593db3c5c57c2c08d1a2 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x8059b1dd8d9895663df44a0fa5f30918 . "./code.cg2js/cg-grid-widget.fasl")
(#x5a8776eb2fd7dbc718c85001ccf2af8d . "./code.cg2js/cg-group-box.fasl")
(#xa6be1969282ba3504d7235a95457902a . "./code.cg2js/cg-header-control.fasl")
(#x59a51bda053e981be1bdbdc5996cba93 . "./code.cg2js/cg-hotspot.fasl")
(#x460aad776826bc4aced7c36dd23345de . "./code.cg2js/cg-html-dialog.fasl")
(#x092c05ff25fd4360c948fea2989c7d25 . "./code.cg2js/cg-html-widget.fasl")
(#x0cf5516c6e03810ba18df8c7958b338a . "./code.cg2js/cg-icon-pixmap.fasl")
(#xc056b41c8fc1c907e39bd84ddbc72f49 . "./code.cg2js/cg-icon.fasl")
(#xce4a124f9ac77f2885cb89c79a03a5e1 . "./code.cg2js/cg-ie.fasl")
(#xafff2e3bc373b3542c1022a20511d429 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0635849960df444ee229a6de8742cfb5 . "./code.cg2js/cg-intersect.fasl")
(#x3811cac1a515850e91b2dcab605c4a7e . "./code.cg2js/cg-item-list.fasl")
(#xb8b8475f5dccaec4a7dfa9d5472d2aff . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xae8cad194a748ba051f55073bdffdc28 . "./code.cg2js/cg-lamp.fasl")
(#x0acc472da54c652d8b9c924858af2e38 . "./code.cg2js/cg-layout.fasl")
(#xf61e11a14df191a5f4e25626260bbe54 . "./code.cg2js/cg-lettered-menu.fasl")
(#xe9253b429019a897d22a2f6d2aafba8b . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe0ec5e91613221372bc15dbf8f71aaab . "./code.cg2js/cg-lisp-text.fasl")
(#xb5621f110690a222d76e1b500cb472e2 . "./code.cg2js/cg-lisp-widget.fasl")
(#x24de5ab86926b3e19a3b4bb938d436e7 . "./code.cg2js/cg-list-view.fasl")
(#x73e87fd2e604ea2f3c1fa56ae25b3e89 . "./code.cg2js/cg-mci.fasl")
(#x0aba82229e147452824a6fa0c4749a1d . "./code.cg2js/cg-menu-tooltip.fasl")
(#xee2d096cf9ef3ff42006fe1e634e1c68 . "./code.cg2js/cg-menu.fasl")
(#xac45ec0403ee3b0161249025fd0f6586 . "./code.cg2js/cg-message-dialog.fasl")
(#x9e20921f4a06d4e1f4ae6559762f8d58 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x09ddb392003bd41666156d30bae90a58 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x18f9516bf9b3e6640d45d1a44d9929dd . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x64d05d36810779a25780e0419fab1b13 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6df3b895ce16a0bdb66c2071f3c3435f . "./code.cg2js/cg-multi-picture-button.fasl")
(#x46814d6debccd1b1b8b171b87c60748a . "./code.cg2js/cg-nodes.fasl")
(#x967433ec20d6de5b663bcd2c09d8eae8 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x2eb2ad8287a7a65d3acf97721f8fe3b5 . "./code.cg2js/cg-object-editor.fasl")
(#x1b352e312cf58820066220aaff29d889 . "./code.cg2js/cg-ocx.fasl")
(#x99b7191ae784b5df752a6f5199120a53 . "./code.cg2js/cg-os-widget.fasl")
(#x15e8b6989b259c36c690735445a69953 . "./code.cg2js/cg-os-window.fasl")
(#x5b86acc7921e87611cffb716be7a4618 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xcf644d2aa39e725151583f51daa9d0af . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xcf3725afb9923ca3561e282e3380f1e9 . "./code.cg2js/cg-outline.fasl")
(#xc694224d29def501fc311f9dc1ffb191 . "./code.cg2js/cg-palette.fasl")
(#xa8f8bf6a5b3246943f7db6a7ccdc06b8 . "./code.cg2js/cg-paren-matching.fasl")
(#x497437e3907707047a5b43a0e064d3e6 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x8f0022b4c9518e77029b478a834d579c . "./code.cg2js/cg-picture-widget.fasl")
(#xe6b2c75c716d3b6f3ee942a87ce7c873 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xac1e47731f2e54b4090e3142e737bbf4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xc2db3dbc09e322b9568537fa7ae1ff68 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x59ba67deac16ad8f4f1c5fddbfbc5562 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa089dd981c3f7d6bea97e44a2d6f368c . "./code.cg2js/cg-pixmap.fasl")
(#xe38726b7e5aee04bf84c62b3048c2e24 . "./code.cg2js/cg-player.fasl")
(#x555a836ef7f6264ee574456253a4186f . "./code.cg2js/cg-plot-widget.fasl")
(#x21cac84c2f21a53acbe97d9b94857048 . "./code.cg2js/cg-printing.fasl")
(#x3ef1159805a179b9e12956fca2426ec2 . "./code.cg2js/cg-progress-indicator.fasl")
(#x2fc58b3f3d06e74fb7108e814bc8bf95 . "./code.cg2js/cg-project-window.fasl")
(#x1b42d9ace41e6fe9508d57128dffc923 . "./code.cg2js/cg-property.fasl")
(#x75419d662c2fb4801aab6cd514ffdf23 . "./code.cg2js/cg-radio-button.fasl")
(#xeb63b5baa78c3b207a3a1327bcee4172 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x09f489436f025b02f3e02f30e90db25a . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x348a6a835aea6766c7bee483bfad4b3e . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x9a7b7f6e21df5e476c965b6aedc75d79 . "./code.cg2js/cg-rich-edit.fasl")
(#x33f54f4ea563ac42ff091898bf49c81b . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe0b6c77efb7af8fe71df8ef692382d18 . "./code.cg2js/cg-scaling-stream.fasl")
(#x331e4528ca57e815fab92f46dba92a12 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x49f3d76f36f8b3c5f6100256f4ff8580 . "./code.cg2js/cg-scroll-bar.fasl")
(#x7a3d5277ad788d847fb081f0cc6c3e31 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x8eca29aa07334e7da1fb7fb0e79858a6 . "./code.cg2js/cg-selected-object.fasl")
(#xfec2f9b05fbc656b3d585e04d1217117 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x7765b86f9c33ad7ba9523d5cf66d94b4 . "./code.cg2js/cg-split-bar.fasl")
(#x9fc8ec7ab9e649801cbcbbf0e421be30 . "./code.cg2js/cg-static-text.fasl")
(#x7c31ea6510a2bfe0367bff9615da9770 . "./code.cg2js/cg-status-bar.fasl")
(#x0bd615bc9657481117dbe0c0a111710d . "./code.cg2js/cg-string-dialog.fasl")
(#xc6f0b95f3899a7b964999fa23ccdad8c . "./code.cg2js/cg-tab-control.fasl")
(#x7300ea350abab4ddd52389ea93f091e5 . "./code.cg2js/cg-template-string.fasl")
(#x4995145ed574c3c4a402a8a2088a79a6 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x2534eda271759da324a0a2ca64b6ad50 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x60c16ca21607bf10ebc242cd5109cf1a . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf5f7620f5289ce67cbf5324fdac9dbd9 . "./code.cg2js/cg-text-or-combo.fasl")
(#x99654efeaeca050f6dff9fa06c1759e9 . "./code.cg2js/cg-text-widget.fasl")
(#xf0f55c78c226e8bb66b4de52be5df0c1 . "./code.cg2js/cg-timer.fasl")
(#x26142af326d644b06680256edf3684e4 . "./code.cg2js/cg-toggling-widget.fasl")
(#x924b549fe2b50509a2356206d28a1d2f . "./code.cg2js/cg-toolbar.fasl")
(#x0e9c65c176990ff60eed834669cbc7d7 . "./code.cg2js/cg-tooltip.fasl")
(#xf414756b4a7819dcd033361f60e8445d . "./code.cg2js/cg-touch.fasl")
(#x8eb25c39023d1daba622a97a75067b3e . "./code.cg2js/cg-trackbar.fasl")
(#x19203e29b38ff055c3c17fcdcc40f397 . "./code.cg2js/cg-tray.fasl")
(#x1f640df4ae94f89137de6f6517b86702 . "./code.cg2js/cg-up-down-control.fasl")
(#x365a9e97234a6094b722c7f97fd55aac . "./code.cg2js/cg-utility-dialog.fasl")
(#x4e6acba188a8587650be88ca241c2c91 . "./code.cg2js/cg-web-browser-dde.fasl")
(#xbe6f5777f9a3cf687a49079ff609ebf4 . "./code.cg2js/cg-web-browser.fasl")
(#x1e4d7e20b4721b6f4f37be85553b8055 . "./code.cg2js/cg-wrap-string.fasl")
(#x6d1adb23b1be614323bc3f4b67f1c145 . "./code.cg2js/cg-yes-no-list.fasl")
(#x942b0732c22cc0a2b9cd09f1516ccc71 . "./code.cg2js/cg.fasl")
(#xc038bf4a302723f27cad4166f1188e25 . "./code.cg2js/cg-yes-no-string.fasl")
(#x0ad25ca45e706c6ac0ac1659dbbeb56c . "./code.idejs/ide-acache.fasl")
(#xf790d307d482bbe536ccba1821fab228 . "./code.idejs/ide-apropos.fasl")
(#x490f8b01485c0258d2d195ce8457095d . "./code.idejs/ide-base.fasl")
(#xc1a7e9d9c7a20a45a3c86365bdd45223 . "./code.idejs/ide-class-browser.fasl")
(#xbdc730c63118096a1f0bde9f0b3977e9 . "./code.idejs/ide-class-grapher.fasl")
(#x49692253309fa364823ee7160327c984 . "./code.idejs/ide-clipboard.fasl")
(#x5ff26885cccd622ae01f0edacd7a767b . "./code.idejs/ide-dev.fasl")
(#xafb443bb2a0d7747292bb5577549a3cb . "./code.idejs/ide-editor.fasl")
(#x0d90152c370cd4c81b56bcc127e408e1 . "./code.idejs/ide-find-definitions.fasl")
(#x0d3e85cab024bfeea5d94e13e3b2864e . "./code.idejs/ide-find-in-files.fasl")
(#x662c2dd2774cb600b0a018b800a1e585 . "./code.idejs/ide-form.fasl")
(#x2b0d67e574cbbc5ec347a45804d03e24 . "./code.idejs/ide-grapher.fasl")
(#x7047d9b2df9d9257e33c2be932ddc9e7 . "./code.idejs/ide-inspector.fasl")
(#x07ba2c3c7a4fba77a6ecba8d4fabd38b . "./code.idejs/ide-intro.fasl")
(#x63dd6c61e374b57f7f77e3ab26527064 . "./code.idejs/ide-keyshort.fasl")
(#x37c09142e646159eabfefc7f4cb646bb . "./code.idejs/ide-navigator.fasl")
(#x4530a76245e5342f01d1ea632d6de0cf . "./code.idejs/ide-package-list.fasl")
(#x47bc89d78441a0b4fb34addd21086906 . "./code.idejs/ide-patch-dialog.fasl")
(#xfa41887f0ae71bda21af121c08cac5df . "./code.idejs/ide-process-browser.fasl")
(#x1b6e39d961164aadd4ed2c978efd604d . "./code.idejs/ide-project.fasl")
(#x1e0297f56e2d72577197b34dea1aae25 . "./code.idejs/ide-trace.fasl")
(#x4c8b6986c83bd1c72bb27b9621d8319f . "./code.idejs/ide-window-list.fasl")
(#xa7261e16ac83f07627bf4f1c4687d6ae . "./code.idejs/ide.fasl")
(#xf954c7d1d43ea0e0cec9ada7b7316935 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
